//PNG Image strip animation viewer
// By: Brett Lajzer
//
// Takes one argument, a text file where the lines are:
//   1 <file path>
//   2 <number of frames in animation>


#include <iostream>
#include <fstream>
#include <string>

#include <SDL/SDL.h>
#include <SDL/SDL_image.h>

SDL_Surface *screen;
SDL_Surface *background;
SDL_Surface *light_grey_block;
SDL_Surface *dark_grey_block;

//makes the background black
void bg_black(){
	SDL_FillRect(background, 0, SDL_MapRGB(background->format, 0, 0, 0));
}

//makes the background white
void bg_white(){
	SDL_FillRect(background, 0, SDL_MapRGB(background->format, 255, 255, 255));
}

//makes the background into photoshop-esque transparency squares
void bg_greys(){
	SDL_Rect temp;

	for(int y = 0; y < background->h; y += 8){
		for(int x = 0; x < background->w; x += 8){
			temp.w = 8;
			temp.h = 8;
			temp.x = x;
			temp.y = y;

			if((y/8) % 2 == 0){
				if((x/8) % 2 == 0){
					SDL_BlitSurface(light_grey_block, &light_grey_block->clip_rect, background, &temp);
				}else{
					SDL_BlitSurface(dark_grey_block, &dark_grey_block->clip_rect, background, &temp);
				}
			}else{
				if((x/8) % 2 == 0){
					SDL_BlitSurface(dark_grey_block, &dark_grey_block->clip_rect, background, &temp);
				}else{
					SDL_BlitSurface(light_grey_block, &light_grey_block->clip_rect, background, &temp);
				}
			}
		}
	}
}

int main(int argc, char *argv[]){
	//if no file is specified, exit with usage tip
	if(argc < 2){
		std::cerr << "USAGE: " << argv[0] << " config-file" << std::endl;
		return 1;
	}

	//open up config file
	std::ifstream input(argv[1]);

	if(!input){
		std::cerr << "ERROR: Cannot locate config file '" << argv[1] << "'!" << std::endl;
		return 1;
	}

	//read in animation file path and number of frames
	std::string anifile;
	int frames;

	input >> anifile;
	input >> frames;

	//init SDL
	SDL_Init(SDL_INIT_TIMER | SDL_INIT_VIDEO);

	//load up the file and check for errors
	SDL_Surface *ani = IMG_Load(anifile.c_str());
	
	if(!ani){
		std::cerr << "ERROR: Cannot locate image '" << anifile << "'!" << std::endl;
		return 1;
	}
	

	Uint32 rmask, gmask, bmask, amask;

	/*SDL interprets each pixel as a 32-bit number, so our masks must depend
	 on the endianness (byte order) of the machine */
	#if SDL_BYTEORDER == SDL_BIG_ENDIAN
	rmask = 0xff000000;
	gmask = 0x00ff0000;
	bmask = 0x0000ff00;
	amask = 0x000000ff;
	#else
	rmask = 0x000000ff;
	gmask = 0x0000ff00;
	bmask = 0x00ff0000;
	amask = 0xff000000;
	#endif

	//create a new screen surface that is exactly as large as one frame of the image
	screen = SDL_SetVideoMode(ani->w / frames, ani->h, 32, SDL_HWSURFACE | SDL_DOUBLEBUF);
	background = SDL_CreateRGBSurface(SDL_HWSURFACE, ani->w / frames, ani->h, 32, rmask, gmask, bmask, amask);

	//setup images for the photoshop-type transparency thingy
	light_grey_block = SDL_CreateRGBSurface(SDL_HWSURFACE, 8, 8, 32, rmask, gmask, bmask, amask);
	 SDL_FillRect(light_grey_block, 0, SDL_MapRGB(light_grey_block->format, 153, 153, 153));
	dark_grey_block = SDL_CreateRGBSurface(SDL_HWSURFACE, 8, 8, 32, rmask, gmask, bmask, amask);
	 SDL_FillRect(dark_grey_block, 0, SDL_MapRGB(dark_grey_block->format, 102, 102, 102));
	

	//variables used in the main loop
	bool done = false;
	bool framebyframe = false;
	Uint32 ticks_begin, ticks_end, tick_diff;
	int current_frame = 0;
	SDL_Rect src;
	SDL_Event e;

	bg_black();

	//main program loop
	while(!done){
		ticks_begin = SDL_GetTicks();

		while(SDL_PollEvent(&e)) {  
			if(e.type == SDL_KEYDOWN){
				if(e.key.state == SDL_PRESSED){
					if(e.key.keysym.sym == SDLK_ESCAPE){
						done = true;
					}
					if(e.key.keysym.sym == SDLK_SPACE){
						framebyframe = !framebyframe;
					}
					if(e.key.keysym.sym == SDLK_LEFT){
						current_frame--;
						if(current_frame < 0){
							current_frame = frames-1;
						}
					}
					if(e.key.keysym.sym == SDLK_RIGHT){
						current_frame++;
						if(current_frame == frames){
							current_frame = 0;
						}
					}
					if(e.key.keysym.sym == SDLK_1){
						bg_black();
					}
					if(e.key.keysym.sym == SDLK_2){
						bg_white();
					}
					if(e.key.keysym.sym == SDLK_3){
						bg_greys();
					}
				}
			}
		}


		//blit the background
		SDL_BlitSurface(background, &background->clip_rect, screen, &screen->clip_rect);

		//setup src rect and blit the animation frame
		src.w = background->w;
		src.h = background->h;
		src.y = 0;
		src.x = (ani->w / frames) * current_frame;

		SDL_BlitSurface(ani, &src, screen, &screen->clip_rect);

		if(!framebyframe){
			current_frame++;
			if(current_frame == frames){
				current_frame = 0;
			}
		}

		//regulate the fps
		ticks_end = SDL_GetTicks();
		tick_diff = ticks_end - ticks_begin;

		if(tick_diff < 33){
			SDL_Delay(33 - tick_diff);
		}

		//flip the screen
		SDL_Flip(screen);
	}

	//close SDL
	SDL_Quit();

	return 0;
}
